@@ -1,10 +1,13 @@
+# Copyright 2016 The Michael Song. All rights rberved.
+# Use of this source code is governed by a BSD-style
+# license that can be found in the LICENSE file.
 
 package Spp::Tools;
 
 use Exporter;
 our @ISA    = qw(Exporter);
 our @EXPORT = qw(to_json from_json error count
-  start_with end_with rev_str
+  start_with end_with rev_str first tail rest subarray
   trim read_file write_file len to_trace_str);
 
 use 5.012;
@@ -14,7 +17,10 @@ use Spp::IsAtom;
 
 sub to_json { 
   my $data = shift;
-  return $data if is_chars($data);
+  if (is_chars($data)) {
+    my $json_str = encode_json([$data]);
+    return substr($json_str, 1, -1);
+  } 
   return encode_json($data);
 }
 
@@ -22,31 +28,72 @@ sub from_json { return decode_json(shift) }
 
 sub error { say @_; exit() }
 
+sub first {
+  my $data = shift;
+  if (is_chars($data)) { return substr($data, 0, 1) }
+  if (is_array($data)) { return $data->[0] }
+  error("could not first($data)");
+}
+
+sub tail {
+  my $data = shift;
+  if (is_chars($data)) { return substr($data, -1) }
+  if (is_array($data)) { return $data->[-1] }
+  error("Could not tail($data)");
+}
+
+sub rest {
+   my $data = shift;
+   return substr($data, 1) if is_chars($data);
+   if (is_array($data)) {
+     my $len_data = len($data);
+     # copy $data, splice would change array
+     my @array = @{$data};
+     return [ splice(@array, 1, $len_data) ];
+   }
+   error("rest only could do str or array");
+}
+
+sub subarray {
+  my ($array, $from, $to) = @_;
+  # make copy of $array, splice would change it
+  my @array = @{$array};
+  if (is_array($array)) {
+    if ($to < 0) {
+      my $len = len($array) + $to - $from + 1;
+      my $sub_array = [ splice @array, $from, $len ];
+      return $sub_array;
+    }
+    return [ splice @array, $from, $to ];
+  }
+  error("subarray only could process array");
+}
+
 sub count {
-   my ( $str, $char ) = @_;
-   my @array = ( $str =~ /$char/g );
+   my ($str, $char) = @_;
+   my @array = ($str =~ /$char/g);
    return scalar(@array);
 }
 
 sub start_with {
-   my ( $str, $start ) = @_;
-   return 1 if index( $str, $start ) == 0;
+   my ($str, $start) = @_;
+   return 1 if index($str, $start) == 0;
    return 0;
 }
 
 sub end_with {
-   my ( $str, $end ) = @_;
-   return start_with( rev_str($str), rev_str($end) );
+   my ($str, $end) = @_;
+   return start_with(rev_str($str), rev_str($end));
 }
 
 sub rev_str {
    my $str = shift;
-   return scalar( reverse($str) );
+   return scalar(reverse($str));
 }
 
 sub trim {
    my $str = shift;
-   if ( is_chars($str) ) {
+   if (is_chars($str)) {
       $str =~ s/^\s+|\s+$//g;
       return $str;
    }
@@ -63,7 +110,7 @@ sub read_file {
 }
 
 sub write_file {
-   my ( $file, $str ) = @_;
+   my ($file, $str) = @_;
    open my ($fh), '>:utf8', $file or die $!;
    print {$fh} $str;
    return $file;
@@ -71,15 +118,15 @@ sub write_file {
 
 sub len {
    my $data = shift;
-   return scalar( @{$data} ) if is_array($data);
+   return scalar(@{$data}) if is_array($data);
    return length($data) if is_chars($data);
 }
 
 sub to_trace_str {
-   my ( $str, $n ) = @_;
+   my ($str, $n) = @_;
    $str = rev_str($str);
    my @buf = ();
-   for my $ch ( split( '', $str ) ) {
+   for my $ch (split('', $str)) {
       given ($ch) {
          when ("\n") { push @buf, 'n\\' }
          when ("\r") { push @buf, 'r\\' }
@@ -87,13 +134,13 @@ sub to_trace_str {
          default     { push @buf, $ch }
       }
    }
-   $str = join( '', @buf );
+   $str = join('', @buf);
    my $len = len($str);
-   if ( $len > $n ) {
-      $str = substr( $str, 0, $n );
+   if ($len > $n) {
+      $str = substr($str, 0, $n);
    }
-   elsif ( $len < $n ) {
-      $str = ( ' ' x ( $n - $len ) ) . $str;
+   elsif ($len < $n) {
+      $str = (' ' x ($n - $len)) . $str;
    }
    return rev_str($str);
 }
